package main

import (
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"path"
	"strconv"
	"strings"
	"time"
)

type SubmissionHandler struct {
	ChallengeEnd      time.Time
	DenyChName        bool
	AllowRegistration bool
}

func (s SubmissionHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	log.Printf("Handling %s request from %s: %s [%s]\n", r.Method, r.RemoteAddr, r.URL.Path, r.UserAgent())

	w.Header().Set("Content-Type", "application/json")

	// Check request type and size
	if r.Method != "POST" {
		http.Error(w, "{\"errmsg\":\"Requête invalide.\"}", http.StatusBadRequest)
		return
	} else if r.ContentLength < 0 || r.ContentLength > 1023 {
		http.Error(w, "{\"errmsg\":\"Requête trop longue ou de taille inconnue\"}", http.StatusRequestEntityTooLarge)
		return
	}

	// Extract URL arguments
	var sURL = strings.Split(r.URL.Path, "/")

	if len(sURL) == 2 && s.AllowRegistration && sURL[1] == "registration" {
		if _, err := os.Stat(path.Join(SubmissionDir, "_registration")); os.IsNotExist(err) {
			log.Println("Creating _registration directory")
			if err := os.MkdirAll(path.Join(SubmissionDir, "_registration"), 0777); err != nil {
				log.Println("Unable to create _registration directory: ", err)
				http.Error(w, "{\"errmsg\":\"Internal server error. Please retry in few seconds.\"}", http.StatusInternalServerError)
				return
			}
		}

		if f, err := ioutil.TempFile(path.Join(SubmissionDir, "_registration"), ""); err != nil {
			log.Println("Unable to open registration file:", err)
			http.Error(w, "{\"errmsg\":\"Internal server error. Please retry in few seconds..\"}", http.StatusInternalServerError)
		} else {
			// Read request body
			var body []byte
			if r.ContentLength > 0 {
				tmp := make([]byte, 1024)
				for {
					n, err := r.Body.Read(tmp)
					for j := 0; j < n; j++ {
						body = append(body, tmp[j])
					}
					if err != nil || n <= 0 {
						break
					}
				}
			}

			f.Write(body)
			f.Close()
		}

		http.Error(w, "{\"errmsg\":\"Demande d'enregistrement acceptée\"}", http.StatusAccepted)
		return
	} else if len(sURL) != 3 {
		http.Error(w, "{\"errmsg\":\"Requête invalide.\"}", http.StatusBadRequest)
		return
	}

	if time.Now().Sub(s.ChallengeEnd) > 0 {
		http.Error(w, "{\"errmsg\":\"Vous ne pouvez plus soumettre, le challenge est terminé.\"}", http.StatusForbidden)
		return
	}

	// Parse arguments
	if _, err := os.Stat(path.Join(TeamsDir, sURL[1])); os.IsNotExist(err) || len(sURL[1]) < 1 || sURL[1][0] == '_' {
		http.Error(w, "{\"errmsg\":\"Requête invalide.\"}", http.StatusBadRequest)
		return
	} else if pex, err := strconv.Atoi(sURL[2]); (s.DenyChName || sURL[2] != "name") && err != nil {
		http.Error(w, "{\"errmsg\":\"Requête invalide.\"}", http.StatusBadRequest)
		return
	} else {
		team := sURL[1]

		var exercice string
		if sURL[2] == "name" {
			exercice = sURL[2]
		} else {
			exercice = fmt.Sprintf("%d", pex)
		}

		if len(exercice) <= 0 {
			log.Println("EMPTY $EXERCICE RECEIVED:", exercice)
			http.Error(w, "{\"errmsg\":\"Internal server error. Please retry in few seconds.\"}", http.StatusInternalServerError)
			return
		}

		if _, err := os.Stat(path.Join(SubmissionDir, team)); os.IsNotExist(err) {
			log.Println("Creating submission directory for", team)
			if err := os.MkdirAll(path.Join(SubmissionDir, team), 0777); err != nil {
				log.Println("Unable to create submission directory: ", err)
				http.Error(w, "{\"errmsg\":\"Internal server error. Please retry in few seconds.\"}", http.StatusInternalServerError)
				return
			}
		}

		// Previous submission not treated
		if _, err := os.Stat(path.Join(SubmissionDir, team, exercice)); !os.IsNotExist(err) {
			http.Error(w, "{\"errmsg\":\"Du calme ! une requête est déjà en cours de traitement.\"}", http.StatusPaymentRequired)
			return
		}

		// Read request body
		var body []byte
		if r.ContentLength > 0 {
			tmp := make([]byte, 1024)
			for {
				n, err := r.Body.Read(tmp)
				for j := 0; j < n; j++ {
					body = append(body, tmp[j])
				}
				if err != nil || n <= 0 {
					break
				}
			}
		}

		// Store content in file
		if file, err := os.Create(path.Join(SubmissionDir, team, exercice)); err != nil {
			log.Println("Unable to open exercice file:", err)
			http.Error(w, "{\"errmsg\":\"Internal server error. Please retry in few seconds.\"}", http.StatusInternalServerError)
			return
		} else {
			file.Write(body)
			file.Close()
		}
		http.Error(w, "{\"errmsg\":\"Son traitement est en cours...\"}", http.StatusAccepted)
	}
}
